import random

## format: m is a matrix of 0 and 1. pp_matrix(m) prints out m in
## matrix form
def pp_matrix(m):
  print('\n'.join([''.join(['{:5,d}'.format(item) for item in row]) 
                   for row in m]))
  return None

## format: i is a integer, graph is an adjacency
## matrix. neighbour(i,graph) returns the list of neighbours of i in
## graph.
def neighbour (i, graph):
  li,n = [],len(graph)
  for j in range(n):
    li += [j] if graph[i][j] == 1 else []
  return li

## format: i is a integer, graph is an adjacency
## matrix. trans_closure(i,graph) returns the list of reachable
## vertices from i in graph with a finite number of steps.
def trans_closure (i, graph):
  clos = []
  sources = [i]
  while sources != []:
    x = sources[0]
    for v in neighbour(x,graph):
      if v not in clos:
        clos += [v]
        sources += [v] if v != i else []
    sources = sources[1:]
  return clos

## format: graph is an adjacency matrix. acyclic(graph) returns True
## if graph is acyclic, False otherwise.
def acyclic (graph):
  n = len(graph)
  for i in range(n):
    if i in trans_closure(i,graph): return False
  return True

## format; path is a string, representing a file path. file2list(path)
## returns the list li, such that li[i] is the list of number appering
## on the ith line of the file at path.
def file2list (path):
  f,li = open(path,'r'),[]
  for line in f:
    li += [[int(k) for k in line.split()]]
  return li

# Careful! update modify m in place.
def update(vot,m):
  for i in range(len(vot)):
    for k in vot[i+1:]:
      m[vot[i]][k] += 1

# li must be non empty!
def duel(li):
  n = len(li[0])
  comp = [[0 for j in range(n)] for i in range(n)]
  for vot in li: update(vot,comp)
  return comp

def score(comp):
  li,n = [],len(comp)
  for i in range(n):
    for j in range(i+1,n):
      cij,cji = comp[i][j],comp[j][i]
      li += [[i,j,cij] if cij>cji else [j,i,cji]]
  return li


## Reminder -- merge sort

# It is just the merge function but in an iterative fashion. (It
# permits to run the merge on big list, as Python has very limited
# features concerning recursion...)
def merge_it(comp,l0,l1):
  li = []
  while(l0 != [] and l1 != []):
    if comp(l0[0],l1[0]):
      li = li + [l0[0]]
      l0 = l0[1:]
    else:
      li = li + [l1[0]]
      l1 = l1[1:]
  return li+l0+l1

# It is the iterative version of the mergesort. See how it is much
# more elaborated than the simpler recursive version.
def mergesort_it(comp,l):
  n = len(l)
  l = map(lambda x: [x],l)
  while n > 1:
    if n % 2 == 1:
      l = l[:-2] + [merge_it(comp,l[-2],l[-1])]
    for i in range(n//2):
      l = l[:i] + [merge_it(comp,l[i],l[i+1])] + l[i+2:]
    n = n//2
  return l[0]

## End of sorting functions

## format: li is a list of triples. custom_sort(li) sorts li with the
## order relation comparing the last coordinate of the triples in
## reverse order.
def custom_sort(li): return mergesort_it(lambda x,y: x[2]>y[2],li)

def tideman(votes):
  nvot,ncand = len(votes),len(votes[0])
  scores = custom_sort(score(duel(votes)))
  graph = [[0 for j in range(ncand)] for i in range(ncand)]
  for (i,j,_) in scores:
    graph[i][j] = 1
    if not acyclic(graph): graph[i][j] = 0
  return graph

def winner(graph):
  n = len(graph)
  for j in range(n):
    if [graph[i][j] for i in range(n)] == [0 for i in range(n)]:
      break
  return j

## TEST ##

# for test in range(1000):
#   graph = [[random.randint(0,1) for j in range(6)] for i in range(6)]
#   for i in range(len(graph)): graph[i][i] = 0
#   if acyclic(graph):
#     print "test", test, "is acyclic:"
#     pp_matrix(graph)

for (i,j,k) in custom_sort(score(duel(file2list("votesFile")))):
  print i, "win over", j, "by", k, "voters"

# pp_matrix (tideman(file2list("../../td9/votesFile")))
# print winner(tideman(file2list("../../td9/votesFile")))
